from flask import Flask, request, jsonify
import subprocess
import os
import signal

app = Flask(__name__)

mlflow_process = None

@app.route('/serve_latest', methods=['POST'])
def trigger_mlflow():
    global mlflow_process

    try:
        data = request.json
        model_uri ='models:/Random Forest Model/latest'
        port = data.get('port', 5001)

        if mlflow_process and mlflow_process.poll() is None:
            os.kill(mlflow_process.pid, signal.SIGTERM)
            mlflow_process = None
            print("Old MLflow process terminated.")

        mlflow_process = subprocess.Popen([
            'mlflow', 'models', 'serve',
            '-m', model_uri,
            '-p', str(port),
            '--host', '0.0.0.0'
        ])
        print(f"New MLflow process started with PID: {mlflow_process.pid}")

        return jsonify({
            "message": "MLflow serving started",
            "model_uri": model_uri,
            "port": port,
            "pid": mlflow_process.pid
        }), 200

    except Exception as e:
        return jsonify({"error": str(e)}), 500


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=8080)
